﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Configuration;
using System.Text;
using KickAppsEmptySample.Model;

namespace KickAppsEmptySample.Provider
{
    public class KickApps
    {

        #region URLS FOR API CALLS
        private const string CreateTokenUrl = "http://api.kickapps.com/rest/token/create/";
        #endregion

        /// <summary>
        /// The site that you are searching. Each affiliate site has a unique as. 
        /// </summary>
        private string siteId;

        /// <summary>
        /// email address or username - required for token request
        /// </summary>
        private string user;

        /// <summary>
        /// developer key - required for token request
        /// </summary>
        private string developerKey;

        /// <summary>
        /// Token object - contains unique TOKEN property generated by KickApps
        /// </summary>
        public Token token;

        /// <summary>
        /// set siteId, user (password or username) and developerKey on object initialisation
        /// </summary>
        public KickApps()
        {
            this.siteId = ConfigurationManager.AppSettings["KickAppsSiteId"];
            this.user = ConfigurationManager.AppSettings["KickAppsUser"];
            this.developerKey = ConfigurationManager.AppSettings["DeveloperKey"];
        }

        private string BuildPostString(Dictionary<string, string> postValues)
        {
            StringBuilder sb = new StringBuilder();
            StringBuilder sbDebug = new StringBuilder();

            foreach (KeyValuePair<string, string> value in postValues)
            {
                if (sb.Length > 0) sb.Append("&");
                sb.Append(string.Format("{0}={1}", value.Key, value.Value));
            }

            return sb.ToString();
        }

        private Result SubmitPostRequest(string url, string postValues, out string response)
        {
            Result result = new Result();
            response = string.Empty;

            try
            {
                ASCIIEncoding encoding = new ASCIIEncoding();
                byte[] data = encoding.GetBytes(postValues);

                HttpWebRequest webRequest = (HttpWebRequest)WebRequest.Create(url);
                webRequest.Method = "POST";
                webRequest.ContentType = "application/x-www-form-urlencoded";
                webRequest.Timeout = System.Threading.Timeout.Infinite;

                webRequest.ContentLength = data.Length;

                Stream requestStream = webRequest.GetRequestStream();
                requestStream.Write(data, 0, data.Length);
                requestStream.Close();
                requestStream.Flush();

                WebResponse webResponse = webRequest.GetResponse();
                StreamReader reader = new StreamReader(webResponse.GetResponseStream());

                var dataReceived = reader.ReadToEnd();

                webResponse.Close();
                reader.Close();
                webRequest.Abort();

                result.returnCode = Result.ReturnCode.Success;
                response = dataReceived;
            }

            catch (Exception ex)
            {
                result.returnCode = Result.ReturnCode.Failure;
                result.errorMessage = ex.Message;
            }


            return result;
        }

        /// <summary>
        /// creates a token object (which contains TOKEN property erquired by all API calls
        /// </summary>
        public void CreateToken()
        {
            // builds the final url that post values will be sent to,
            // if you had a look at the REST documentation on kickapps.com, 
            // you will notice that create token url format is 
            // http://api.kickapps.com/rest/token/create/{username}/{as} or http://api.kickapps.com/rest/token/create/{email}/{as}
            StringBuilder url = new StringBuilder();
            url.Append(string.Format("{0}", CreateTokenUrl));
            url.Append(string.Format("{0}", user));
            url.Append(string.Format("/{0}", siteId));

            // now specify all the post parameters you want to sent to the url you created above
            // according to the API documentation (http://www.kickapps.com/documentation/index.php/Create_Token)
            // CreateToken API call requires developerKey, and 3 other optional parameters
            Dictionary<string, string> postValues = new Dictionary<string, string>();
            postValues.Add("developerKey", developerKey);

            // response will contain a JSON response
            string response;
            var header = SubmitPostRequest(url.ToString(), BuildPostString(postValues), out response);

            if (header.returnCode == Result.ReturnCode.Success)
            {
                // convert JSON response to our Model/Token oject 
                token = (Token)Newtonsoft.Json.JsonConvert.DeserializeObject(response, typeof(Token));
            }

        }

    }
}